@@ -7,10 +7,10 @@ git-buildpackage and friends
 """
 
 import subprocess
-import sys
 import os
 import os.path
 import signal
+import log
 from errors import GbpError
 
 class CommandExecFailed(Exception):
@@ -23,13 +23,12 @@ class Command(object):
     Wraps a shell command, so we don't have to store any kind of command line options in 
     one of the git-buildpackage commands
     """
-    verbose = False
-
     def __init__(self, cmd, args=[], shell=False, extra_env=None):
         self.cmd = cmd
         self.args = args
         self.run_error = "Couldn't run '%s'" % (" ".join([self.cmd] + self.args))
         self.shell = shell
+        self.retcode = 1
         if extra_env is not None:
             self.env = os.environ.copy()
             self.env.update(extra_env)
@@ -42,8 +41,7 @@ class Command(object):
             "restore default signal handler (http://bugs.python.org/issue1652)"
             signal.signal(signal.SIGPIPE, signal.SIG_DFL)
 
-        if self.verbose:
-            print self.cmd, self.args, args
+        log.debug("%s %s %s" % (self.cmd, self.args, args))
         cmd = [ self.cmd ] + self.args + args
         if self.shell: # subprocess.call only cares about the first argument if shell=True
             cmd = " ".join(cmd)
@@ -53,25 +51,26 @@ class Command(object):
         """
         run self.cmd adding args as additional arguments
 
-        be verbose about errors and encode them in the return value, don't pass
-        on exceptions
+        Be verbose about errors and encode them in the return value, don't pass
+        on exceptions.
         """
         try:
             retcode = self.__call(args)
             if retcode < 0:
-                print >>sys.stderr, "%s was terminated by signal %d" % (self.cmd,  -retcode)
+                log.err("%s was terminated by signal %d" % (self.cmd,  -retcode))
             elif retcode > 0:
-                print >>sys.stderr, "%s returned %d" % (self.cmd,  retcode)
+                log.err("%s returned %d" % (self.cmd,  retcode))
         except OSError, e:
-            print >>sys.stderr, "Execution failed:", e
+            log.err("Execution failed: " + e.__str__())
             retcode = 1
         if retcode:
-            print >>sys.stderr, self.run_error
+            log.err(self.run_error)
+        self.retcode = retcode
         return retcode
 
     def __call__(self, args=[]):
-        """run the command, convert all errors into CommandExecFailed, assumes
-        that the lower levels printed an error message - only usefull if you
+        """Run the command, convert all errors into CommandExecFailed, assumes
+        that the lower levels printed an error message - only useful if you
         only expect 0 as result
         >>> Command("/bin/true")(["foo", "bar"])
         >>> Command("/foo/bar")()
@@ -160,6 +159,16 @@ class RepackTarArchive(Command):
         self.run_error = 'Couldn\'t repack "%s"' % self.archive
 
 
+class CatenateTarArchive(Command):
+    """Wrap tar to catenate a tar file with the next"""
+    def __init__(self, archive, **kwargs):
+        self.archive = archive
+        Command.__init__(self, 'tar', ['-A', '-f', archive], **kwargs)
+
+    def __call__(self, target):
+        Command.__call__(self, [target])
+
+
 class RemoveTree(Command):
     "Wrap rm to remove a whole directory tree"
     def __init__(self, tree):
@@ -198,6 +207,7 @@ class GitCommand(Command):
         self.run_error = "Couldn't run git %s" % cmd
 
 
+# FIXME: move to gbp.git.__init__
 class GitInit(GitCommand):
     """Wrap git init"""
     def __init__(self):
@@ -205,20 +215,14 @@ class GitInit(GitCommand):
         self.run_error = "Couldn't init git repository"
 
 
+# FIXME: move to gbp.git.__init__
 class GitClone(GitCommand):
     """Wrap git clone"""
     def __init__(self):
         GitCommand.__init__(self, 'clone')
         self.run_error = "Couldn't clone git repository"
 
-
-class GitShowBranch(GitCommand):
-    """Wrap git show-branch"""
-    def __init__(self):
-        GitCommand.__init__(self, 'branch')
-        self.run_error = "Couldn't list branches"
-
-
+# FIXME: move to  gbp.git.create_branch
 class GitBranch(GitCommand):
     """Wrap git branch"""
     def __init__(self):
@@ -232,21 +236,7 @@ class GitBranch(GitCommand):
         GitCommand.__call__(self, options)
 
 
-class GitCheckoutBranch(GitCommand):
-    """Wrap git checkout in order tos switch to a certain branch"""
-    def __init__(self, branch):
-        GitCommand.__init__(self, 'checkout', [branch])
-        self.branch = branch
-        self.run_error = 'Couldn\'t switch to branch "%s"' % self.branch
-
-
-class GitPull(GitCommand):
-    """Wrap git pull"""
-    def __init__(self, repo, branch):
-        GitCommand.__init__(self, 'pull', [repo, branch]) 
-        self.run_error = 'Couldn\'t pull "%s" to "%s"' % (branch, repo)
-
-
+# FIXME: move to gbp.git.fetch
 class GitFetch(GitCommand):
     """Wrap git fetch"""
     def __init__(self, remote = None):
@@ -256,6 +246,7 @@ class GitFetch(GitCommand):
         GitCommand.__init__(self, 'fetch', opts)
 
 
+# FIXME: move to gbp.git.merge
 class GitMerge(GitCommand):
     """Wrap git merge"""
     def __init__(self, branch, verbose=False):
@@ -264,6 +255,7 @@ class GitMerge(GitCommand):
         self.run_error = 'Couldn\'t merge from "%s"' % (branch,)
 
 
+# FIXME: move to gbp.git.create_tag
 class GitTag(GitCommand):
     """Wrap git tag"""
     def __init__(self, sign_tag=False, keyid=None):
@@ -286,6 +278,7 @@ class GitTag(GitCommand):
         GitCommand.__call__(self, cmd)
 
 
+# FIXME: move to gbp.git.add
 class GitAdd(GitCommand):
     """Wrap git add to add new files"""
     def __init__(self, extra_env=None):
@@ -293,26 +286,6 @@ class GitAdd(GitCommand):
         self.run_error = "Couldn't add files"
 
 
-class GitRm(GitCommand):
-    """Wrap git rm to remove files"""
-    def __init__(self, verbose=False):
-        args = [ ['--quiet'], [] ][verbose]
-        GitCommand.__init__(self, cmd='rm', args=args)
-        self.run_error = "Couldn't remove files"
-
-
-class GitCommitAll(GitCommand):
-    """Wrap git commit to commit all changes"""
-    def __init__(self, verbose=False, **kwargs):
-        args = ['-a'] + [ ['-q'], [] ][verbose]
-        GitCommand.__init__(self, cmd='commit', args=args, **kwargs)
-
-    def __call__(self, msg=''):
-        args = [ [], ['-m', msg] ][len(msg) > 0]
-        self.run_error = "Couldn't %s %s" % (self.cmd, " ".join(self.args + args))
-        GitCommand.__call__(self, args)
-
-
 def copy_from(orig_dir, filters=[]):
     """
     copy a source tree over via tar
