@@ -89,10 +89,11 @@ class GbpOptionParser(OptionParser):
                  'ignore-regex'    : '',
                  'compression'     : 'auto',
                  'compression-level': '9',
-                 'remote-url-pattern' : 'ssh://alioth.debian.org/git/collab-maint/%(pkg)s.git',
+                 'remote-url-pattern' : 'ssh://git.debian.org/git/collab-maint/%(pkg)s.git',
                  'multimaint'      : 'True',
                  'multimaint-merge': 'False',
                  'pbuilder'        : 'False',
+                 'qemubuilder'     : 'False',
                  'dist'            : 'sid',
                  'arch'            : '',
                  'interactive'     : 'True',
@@ -153,6 +154,8 @@ class GbpOptionParser(OptionParser):
                   "Build for this distribution when using git-pbuilder, default is '%(dist)s'",
              'arch':
                   "Build for this architecture when using git-pbuilder, default is '%(arch)s'",
+             'qemubuilder':
+                  "Invoke git-pbuilder with qemubuilder for building, default is '%(qemubuilder)s'",
              'interactive':
                   "Run command interactive, default is '%(interactive)s'",
              'color':
